#include "cursor.h"
#include "dout.h"

void TkTheme_CursorCleanup (TkTheme_Cursor *wPtr) {
 /*TODO perhaps Tcl_DestroyCommand on wPtr->path or wPtr->pathCmd*/
 /*TODO free pixmaps */
 if (wPtr->data) {
  free (wPtr->data);
 }
 ckfree (wPtr);
}

int TkTheme_CursorBuildPixmapFromData (Tcl_Interp *interp, TkTheme_Cursor *wPtr) {
 if (XpmCreatePixmapFromBuffer (wPtr->dis, wPtr->xWin, wPtr->data, &wPtr->pix, &wPtr->pixMask, NULL)) {
  Tcl_SetResult (interp, "bad XPM", TCL_STATIC);
  return TCL_ERROR;
 }
 return TCL_OK;
}

int TkTheme_CursorSetWindowSize (Tcl_Interp *interp, TkTheme_Cursor *wPtr) {
 Window root;
 int ijunk;
 unsigned int uijunk;
 Status s;

 s = XGetGeometry (wPtr->dis, wPtr->pix, &root, 
  &ijunk, &ijunk, 
  &wPtr->width, &wPtr->height,
  &uijunk, &uijunk
 );

 if (!s) {
  Tcl_SetResult (interp, "unable to fetch geometry of pixmap", TCL_STATIC);
  return TCL_ERROR;
 }

 XResizeWindow (wPtr->dis, wPtr->xWin, wPtr->width, wPtr->height);
 return TCL_OK;
}

int TkTheme_CursorShapeWindow (Tcl_Interp *interp, TkTheme_Cursor *wPtr) {
 int eventBase;
 int errorBase;

 if (!XShapeQueryExtension (wPtr->dis, &eventBase, &errorBase)) {
  Tcl_SetResult (interp, "Your display doesn't appear to support the X shape extension.", TCL_STATIC);
  return TCL_ERROR;
 }

 XShapeCombineMask (wPtr->dis, wPtr->xWin, ShapeBounding, 0, 0, wPtr->pixMask, ShapeSet);
 return TCL_OK;
}

int TkTheme_CursorInstanceCmd OBJ_CMD_ARGS {
 TkTheme_Cursor *wPtr = (TkTheme_Cursor *) clientData;
 char *subCmd;
 int x;
 int y;
 
 /*.path move $x $y*/
 if (objc < 2) {
  Tcl_WrongNumArgs (interp, 1, objv, "hide|move|show ?x ?y");
  return TCL_ERROR;
 }
 
 subCmd = Tcl_GetString (objv[1]);

 if ((2 == objc) && ('h' == subCmd[0])) {
  XUnmapWindow (wPtr->dis, wPtr->xWin);
 } else if ((4 == objc) && ('m' == subCmd[0])) {
  if (TCL_OK != Tcl_GetIntFromObj (interp, objv[2], &x)) {
   return TCL_ERROR;
  }
  if (TCL_OK != Tcl_GetIntFromObj (interp, objv[3], &y)) {
   return TCL_ERROR;
  }
  XMoveWindow (wPtr->dis, wPtr->xWin, x + 1, y + 1);
 } else if ((2 == objc) && ('s' == subCmd[0])) {
  XMapRaised (wPtr->dis, wPtr->xWin);
 } else {
  Tcl_AppendResult (interp, "invalid subcommand ", subCmd, NULL);
  return TCL_ERROR;
 }

 XFlush (wPtr->dis);
 
 return TCL_OK;
}

int TkTheme_CursorCreateWindow (Tcl_Interp *interp, TkTheme_Cursor *wPtr) {
 XSetWindowAttributes xattr;

 xattr.override_redirect = 1;

 wPtr->xWin = XCreateWindow (wPtr->dis, wPtr->root,
  1, 1, 1, 1, 
  0, wPtr->depth,
  InputOutput,
  CopyFromParent,
  CWOverrideRedirect, &xattr
 );

 if (None == wPtr->xWin) {
  Tcl_SetResult (interp, "unable to create window", TCL_STATIC);
  TkTheme_CursorCleanup (wPtr);
  return TCL_ERROR;
 }
 return TCL_OK;
}

void TkTheme_CursorGetInstanceName (Tcl_Interp *interp, char *buf, int len) {
 int i = 0;
 Tcl_CmdInfo cmdInfo;
 loop {
  snprintf (buf, len, "#cursor%d", i);
  if (0 == (Tcl_GetCommandInfo (interp, buf, &cmdInfo))) {
   return;  
  }
  ++i;
 }
}

int TkTheme_CursorCmd OBJ_CMD_ARGS {
 Tk_Window tkm;
 TkTheme_Cursor *wPtr;
 char instName[20];
 Tcl_Obj *res;
 
 /*TkTheme.cursor xpmData*/
 if (2 != objc) {
  Tcl_WrongNumArgs (interp, 1, objv, "xpmData");
  return TCL_ERROR;
 }

 wPtr = (TkTheme_Cursor *) ckalloc (sizeof (TkTheme_Cursor));
 memset (wPtr, 0, sizeof (TkTheme_Cursor));

 tkm = Tk_MainWindow (interp);
 Tk_MakeWindowExist (tkm);

 wPtr->dis = Tk_Display (tkm);
 wPtr->screenNum = Tk_ScreenNumber (tkm); 
 wPtr->depth = DefaultDepth (wPtr->dis, wPtr->screenNum);
 wPtr->root = RootWindow (wPtr->dis, wPtr->screenNum);

#define ERR_CHECK(_func_) if (TCL_OK != _func_) { TkTheme_CursorCleanup (wPtr); return TCL_ERROR; }
 ERR_CHECK (TkTheme_CursorCreateWindow (interp, wPtr));
 
 wPtr->data = safeStrdup (Tcl_GetString (objv[1]));
 wPtr->drawGC = XCreateGC (wPtr->dis, wPtr->xWin, 0, NULL);
 wPtr->drawPending = 0;
 wPtr->pix = None;
 wPtr->pixMask = None;
 wPtr->width = 1;
 wPtr->height = 1;

 ERR_CHECK (TkTheme_CursorBuildPixmapFromData (interp, wPtr));
 ERR_CHECK (TkTheme_CursorSetWindowSize (interp, wPtr));
 ERR_CHECK (TkTheme_CursorShapeWindow (interp, wPtr));
 ERR_CHECK (TkTheme_CursorCreateBlank (interp, wPtr));
#undef ERR_CHECK 



 /*TODO store Tcl_Command token for faster DestroyCommand usage*/
 
 TkTheme_CursorGetInstanceName (interp, instName, 20);

 wPtr->cmd = Tcl_CreateObjCommand (interp, instName, TkTheme_CursorInstanceCmd, 
  (ClientData) wPtr, (Tcl_CmdDeleteProc *) NULL
 );

 XSelectInput (wPtr->dis, wPtr->xWin, (EnterWindowMask | ExposureMask | StructureNotifyMask));

 Tk_CreateGenericHandler (TkTheme_CursorEvent, (ClientData) wPtr);

 XMapRaised (wPtr->dis, wPtr->xWin);
  
 res = Tcl_NewStringObj (instName, -1);
 Tcl_IncrRefCount (res);
 Tcl_SetObjResult (interp, res);
 Tcl_DecrRefCount (res); 

 return TCL_OK;
}
