
#include "theme.h"
#include "label.h"

void ThemeLabelDraw (ThemeLabel *wPtr) {
 if (!wPtr->noBGPixmap) {
  Tk_SetWindowBackgroundPixmap (wPtr->tkwin, ParentRelative); 
 }
 XCopyArea (wPtr->dis, wPtr->pix, wPtr->xWin, wPtr->copyGC, 0, 0, wPtr->width, wPtr->height, 0, 0);
}

static int ThemeLabelInstanceCmd OBJ_CMD_ARGS {
 return TCL_OK;
}

char *ThemeLabelVariableWrite (
 ClientData clientData, Tcl_Interp *interp, 
 char *part1, char *part2, int flags) 
{
 ThemeLabel *wPtr = (ThemeLabel *) clientData;
 char *str;
 
 if (NULL == part2) {
  str = Tcl_GetVar (interp, part1, flags);
 } else {
  str = Tcl_GetVar2 (interp, part1, part2, flags);
 }

 if (NULL == str) {
  str = "";
 }

 ThemeLabelSetText (wPtr, str);
 return NULL; 
}

int ThemeLabelCmd OBJ_CMD_ARGS {
 Tk_Window tkm;
 Tk_Window tkwin;
 ThemeLabel *wPtr;
 char *text = "";
 int i;
 
 if (objc < 2) {
  Tcl_WrongNumArgs (interp, 1, objv, ".path");
  return TCL_ERROR;
 }

 wPtr = (ThemeLabel *) ckalloc (sizeof (ThemeLabel));
 memset (wPtr, 0, sizeof (ThemeLabel));
 
 wPtr->path = safeStrdup (Tcl_GetString (objv[1]));
 
 for (i = 2; i < objc; ++i) {
  int error = 0;
  char *arg = Tcl_GetString (objv[i]);
  if (0 == (strncmp (arg, "-textvariable", 6))) {
   ++i;
   if (i >= objc) {
    Tcl_SetResult (interp, "no value for -textvariable", TCL_STATIC);
    error = 1;
   }
   wPtr->varName = safeStrdup (Tcl_GetString (objv[i]));
  } else if (0 == (strncmp (arg, "-text", 5))) {
   ++i;
   if (i >= objc) {
    Tcl_SetResult (interp, "no value for -text", TCL_STATIC);
    error = 1;
   }
   text = Tcl_GetString (objv[i]);
  } else if (0 == (strncmp (arg, "-nobgpixmap", 5))) {
   ++i;
   if (i >= objc) {
    Tcl_SetResult (interp, "no value for -nobgpixmap", TCL_STATIC);
    error = 1;
   }
   if (TCL_OK != Tcl_GetBooleanFromObj (interp, objv[i], &wPtr->noBGPixmap)) {
    error = 1;
   }
  } else {
   Tcl_SetResult (interp, "invalid argument", TCL_STATIC);
   error = 1;
  }
   
  if (error) {
   ThemeLabelCleanupStructure (wPtr);
   return TCL_ERROR;
  }
 }

 tkm = Tk_MainWindow (interp);
 Tk_MakeWindowExist (tkm);
 
 tkwin = Tk_CreateWindowFromPath (interp, tkm, wPtr->path, NULL);
 if (NULL == tkwin) {
  ThemeLabelCleanupStructure (wPtr);
  return TCL_ERROR;
 }
 
 Tk_MakeWindowExist (tkwin);
 Tk_SetClass (tkwin, "ThemeLabel");
 
 wPtr->tkwin = tkwin;
 wPtr->dis = Tk_Display (tkwin);
 wPtr->drawPending = 0;
 wPtr->xWin = Tk_WindowId (tkwin);
 wPtr->screen = DefaultScreen (wPtr->dis);
 wPtr->colormap = DefaultColormap (wPtr->dis, wPtr->screen);
 wPtr->depth = DefaultDepth (wPtr->dis, wPtr->screen);
 wPtr->tkfont = NULL;
 wPtr->numChars = 0;
 wPtr->fgCol.pixel = 0;
 wPtr->eventMask = (ExposureMask | StructureNotifyMask);
 wPtr->tkfont = Tk_GetFont (NULL, wPtr->tkwin, "Lucida 12");

 Tk_GeometryRequest (tkwin, 1, 1);

 Tcl_CreateObjCommand (interp, wPtr->path, 
  ThemeLabelInstanceCmd, (ClientData) wPtr, (Tcl_CmdDeleteProc *) NULL
 );

 ThemeLabelSetText (wPtr, text);

 /*initilize the Pixmaps and GCs*/
 ThemeLabelEventConfigure (wPtr);
 
 Tk_CreateEventHandler (tkwin, wPtr->eventMask, ThemeLabelEvent, (ClientData) wPtr);

 if (NULL != wPtr->varName) {
  if (TCL_OK != Tcl_TraceVar (interp, wPtr->varName, (TCL_TRACE_WRITES | TCL_GLOBAL_ONLY | TCL_TRACE_ARRAY), \
   ThemeLabelVariableWrite, (ClientData) wPtr)
  ) {
   return TCL_ERROR;
  }
  ThemeLabelVariableWrite (wPtr, interp, wPtr->varName, NULL, 0);
 }
 
 Tcl_SetResult (interp, wPtr->path, TCL_VOLATILE);

 return TCL_OK;
}
