
if 0 {
Copyright 2003 George Peter Staplin
You may use this under the same terms as Tcl.

A Little Make Replacement (AKA tickmake)

Make is a common build tool.  It basically checks the [file mtime] of dependencies
and compares them to the [file mtime] of target files (if they exist).  If a dependency
is newer than the target then the target should be generated.  Most make programs are
complex.  They usually implement a limited-command-language that is difficult to work
with.  Make also has a lot of historical baggage...  With Tcl we have a great command
language that is well suited for a tool like this.
 
Do not be mislead by the small amount of code to do this.  It really is this simple!

If you have the desire to modify this on a Wiki page please append your changed version
or your changes and keep the original intact.

Tested with Tcl 8.4.1 in [OpenBSD].

Thanks to [Gerald Lester] for debugging.

}

proc file.newer? {f _than_ f2} {
 if {![file exists $f] || ![file exists $f2]} {
  return 1
 }
 if {[file mtime $f] > [file mtime $f2]} {
  return 1
 } 
 return 0
}

proc is.file? f {
 return [file isfile $f]
}

proc is.target? t {
 return [info exists ::targetAr($t)]
}

array set ::targetAr {}

proc make {t _from_ fromList _using_ body} {
 set ::targetAr($t) [list $fromList $body]
 interp alias {} $t {} make.target $t
}

proc make.target t {
 if {[is.target? $t]} {
  foreach {fromList body} [set ::targetAr($t)] break

  set build 0
  foreach f $fromList {
   if {[is.target? $f]} {
    set r [make.target $f]
    if {[is.file? $f] && [file.newer? $f than $t]} {
     set build 1
    } elseif {$r} {
     set build 1
    }
   } elseif {[is.file? $f]} {
    if {!$build} {
     set build [file.newer? $f than $t]
    }
   } else {
    return -code error "$f isn't a target or existing source"
   }
  }
  if {$build} {
   uplevel #0 $body
   return 1
  }
  return 0
 } else {
  return -code error "make.target called with an invalid target: $t"
 }
}

proc mexec args {
 set cmd [join $args]
 puts $cmd
 catch {eval exec -- $cmd} msg
 if {[string length $msg]} {
  puts $msg
 }
}

if 0 {
We will now give examples of usage.  
It's quite easy to use, and everything is evaluated in level 0 (global).
It passes all of these tests for me.  You can play around with the touch
command to see that it properly generates targets when needed.
}

set ::CC gcc
set ::CFLAGS "-fPIC -Wall -I/usr/X11R6/include -I/usr/local/tcltk8.4/include"
set ::CSOURCES [list cursor.c cursor_create_blank.c cursor_draw.c cursor_event.c \
cursor_schedule_draw.c dout.c label.c label_cleanup.c label_create_clipmask.c \
label_event.c label_schedule_draw.c label_set_text.c theme.c]
set ::LFLAGS "-L/usr/X11R6/lib -lX11 -lXext -lXpm -L/usr/local/tcltk8.4/lib -ltcl84 -lm"

foreach f $::CSOURCES {
 set froot [file rootname $f]

 set fromList [list]
 lappend fromList $f

 if {[file exists $froot.h]} {
  lappend fromList $froot.h  
 }
 make $froot.o from $fromList using [list mexec $CC $CFLAGS -c $f]
 lappend ::OBJECTLIST $froot.o
}

make TkTheme.so from $::OBJECTLIST using {
 mexec $CC -shared [glob *.o] $::LFLAGS -o TkTheme.so
}

proc clean {} {
 foreach f [glob -nocomplain *.o *.core *.so] {
  file delete $f
 }
}

proc main {} {
 if {0 == $::argc} {
  TkTheme.so
 } elseif {1 == $::argc} {
  eval [lindex $::argv 0]
 } else {
  return -code error "invalid number of arguments: $::argv0 ?target?"
 }
}
main
